/********************************************************************************
 * Copyright (c) Erik Kunze 1995, 1996
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 *
 * changed by EKU
 *******************************************************************************/

#ifdef DEBUG

static char rcsid[] = "$Id: debug.c,v 3.0 1996/10/03 10:23:45 erik Exp $";

/********************************************************************************
 * debug.c - All the debugging stuff.
 *******************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "ZXConfig.h"
#include "z80.h"
#include "mem.h"
#include "dis.h"
#include "debug.h"

/********************************************************************************
 * Defines
 *******************************************************************************/

/********************************************************************************
 * Forwards
 *******************************************************************************/

/********************************************************************************
 * Globals
 *******************************************************************************/

int Debug = 0;			/* Switch on debugging */

/********************************************************************************
 * DumpCpuState() - Print out the CPU registers.
 *
 * Input:  -
 *
 * Return: -
 *******************************************************************************/

void
DumpCpuState(void)
{
	fprintf(stderr, "Z80 CPU State\n-------------\n");

	Z80_StoreFlags();

	fprintf(stderr, "  AF  = %04X\t\t  AF' = %04X\t\t  A = %02X  F = %02X\n",
			AF, AF1, A, F);
	fprintf(stderr, "  BC  = %04X\t\t  BC' = %04X\t\t  B = %02X  C = %02X\n",
			BC, BC1, B, C);
	fprintf(stderr, "  DE  = %04X\t\t  DE' = %04X\t\t  D = %02X  E = %02X\n",
			DE, DE1, D, E);
	fprintf(stderr, "  HL  = %04X\t\t  HL' = %04X\t\t  H = %02X  L = %02X\n",
			HL, HL1, H, L);
	fprintf(stderr, "  IX  = %04X\t\t  IY  = %04X\n\n",	IX, IY);

	fprintf(stderr, "  PC  = %04X\t\t  SP  = %04X\n", PC, SP);

	fprintf(stderr, "   R  = %02X\t\t   I  = %02X\n", R, I);

	fprintf(stderr, "  interrupts %s, IM%d\n",
			IFF1 ? "enabled" : "disabled", IM);

	fprintf(stderr, "  FLAGS: C=%d N=%d P/V=%d H=%d Z=%d S=%d\n\n",
			C_FLAG, N_FLAG, P_FLAG, H_FLAG, Z_FLAG, S_FLAG);
}

/********************************************************************************
 * DebugZ80() - This one is for debugging the Z80 emulation.
 *
 * Input:  -
 *
 * Return: -
 *******************************************************************************/

void
DebugZ80(void)
{
	char buf[80];
	uns16 addr;
	int i;

	if (Debug)
	{
		if (GETCFG(debug) & D_CPU)
		{
			/* print out cpu state */
			DumpCpuState();
		}

		if (GETCFG(debug) & D_SYSVAR)
		{
			/* print out system vars */
			for(addr = 0x5c00; addr < 0x5d00;)
			{
				fprintf(stderr, "%04x ", addr);
				for (i = 0; i < 16; i++)
				{
					fprintf(stderr," %02x", RD_BYTE(addr++));
				}
				fputc('\n', stderr);
			}
			fputc('\n', stderr);
		}
		
		if (GETCFG(debug) & D_MNEMO)
		{
			/* print out assembler mnemonic */
			Disassemble(buf, PC);
			fprintf(stderr, "%s\n\n", buf);
		}
	}
}

#endif /* DEBUG */

/* EOF *************************************************************************/
