//	Copyright (c) Jens Kilian 1998
//
//	Permission to use, copy, modify, distribute, and sell this software and its
//	documentation for any purpose is hereby granted without fee, provided that
//	the above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation, and that the name of the copyright holder not be used in
//	advertising or publicity pertaining to distribution of the software without
//	specific, written prior permission.  The copyright holder makes no
//	representations about the suitability of this software for any purpose.  It
//	is provided "as is" without express or implied warranty.
//
//	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
//	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
//	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
//	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
//	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//	PERFORMANCE OF THIS SOFTWARE.

//	A BFile look-alike which uses the search path.

#include <stdio.h>

#include <DataIO.h>

#include "ZXFile.h"
#include "ZXMime.h"
#include "util.h"

ZXFile::ZXFile(char *pName, char *pMode)
:	mpFile(OpenXzxFile(pName, pMode, ZXMimeTypes::kAnyFile))
{
	// empty
}

ZXFile::~ZXFile(void)
{
	if (mpFile != NULL) {
		fclose(mpFile);
		mpFile = NULL;
	}
}

ssize_t
ZXFile::Read(void *pBuffer, size_t size)
{
	if (mpFile == NULL) {
		return -1;
	}
	return fread(pBuffer, 1, size, mpFile);
}

ssize_t
ZXFile::ReadAt(off_t position, void *pBuffer, size_t size)
{
	if (mpFile == NULL || fsetpos(mpFile, &position) != 0) {
		return -1;
	}
	ssize_t result = fread(pBuffer, 1, size, mpFile);
	(void)fsetpos(mpFile, &position);
	return result;
}

ssize_t
ZXFile::Write(const void *pBuffer, size_t size)
{
	if (mpFile == NULL) {
		return -1;
	}
	return fwrite(pBuffer, 1, size, mpFile);
}

ssize_t
ZXFile::WriteAt(off_t position, const void *pBuffer, size_t size)
{
	if (mpFile == NULL || fsetpos(mpFile, &position) != 0) {
		return -1;
	}
	ssize_t result = fwrite(pBuffer, 1, size, mpFile);
	(void)fsetpos(mpFile, &position);
	return result;
}

off_t
ZXFile::Seek(off_t position, uint32 seekMode)
{
	if (mpFile == NULL || _fseek(mpFile, position, seekMode) != 0) {	// Arrgh.
		return -1;
	}
	return Position();
}

off_t
ZXFile::Position(void) const
{
	fpos_t position;
	if (mpFile == NULL || fgetpos(mpFile, &position) != 0) {
		return -1;
	}
	return position;
}
